from data_manager import DataManager


def fetch_data(dataset_name):
    """fetches data from server

    Args:
        dataset_name (str): Name of dataset

    returns:
        tuple of structure data and response data
    """
    manager = DataManager()
    return manager.fetch_data(dataset_name)


def list_data():
    """list datasets on server

    returns:
        list of all dataset names
    """
    manager = DataManager()
    return manager.list_dataset_names()


def push_data(dataset_name, structure_data, data_source, parameters,
              response_data=None, authors=None, description=None,
              other_metadata=None):
    """push data up to server

    Args:
        dataset_name (str): name of dataset
        structure_data (ND array): materials structure data
        data_source (str): information about source of data
        parameters (dict): parameters used to create the data
        response_data (ND array, optional): materials response data
        authors (str, optional): authors
        description (str, optional): description of dataset
        other_metadata (str, optional): other metadata
    """
    manager = DataManager()
    manager.push_data(dataset_name, structure_data, data_source,
                      parameters, response_data=response_data,
                      authors=authors, description=description,
                      other_metadata=other_metadata)


def get_version():
    from pkg_resources import get_distribution, DistributionNotFound

    try:
        version = get_distribution(__name__).version
    except DistributionNotFound:
        version = "unknown, try running `python setup.py egg_info`"

    return version


__version__ = get_version()

__all__ = ['__version__', 'fetch_data', 'DataManager', 'list_data',
           'push_data']



from pymongo import MongoClient
from bson.binary import Binary
import cPickle as pickle


class DataManager(object):

    def __init__(self, url=None, port=None):
        self.port = port
        self.url = url
        if url is None and port is None:
            client = MongoClient()
        elif url is not None and port is not None:
            client = MongoClient(url + str(port))
        else:
            raise RuntimeError('url and port must both be specified')
        self.client = client

    def fetch_data(self, dataset_name):
        db = self.client.pymks
        self.data_dict = db.data.find({"dataset_name": dataset_name})[0]
        X = pickle.loads(self.data_dict['structure_data'])
        y = pickle.loads(self.data_dict['response_data'])
        return X, y

    def list_dataset_names(self):
        db = self.client.pymks
        collections = db.metadata.find()
        return [c['dataset_name'] for c in collections]

    def push_data(self, dataset_name, structure_data, data_source, parameters,
                  response_data=None, authors=None, description=None,
                  other_metadata=None):
        db = self.client.pymks
        bi_structure_data = Binary(pickle.dumps(structure_data, protocol=2))
        bi_response_data = Binary(pickle.dumps(response_data, protocol=2))
        data_dict = {'dataset_name': dataset_name,
                     'structure_data': bi_structure_data,
                     'response_data': bi_response_data}
        db.data.insert_one(data_dict)
        meta_data_dict = {'dataset_name': dataset_name,
                          'data_source': data_source,
                          'parameters': parameters,
                          'authors': authors,
                          'description': description,
                          'other_metadata': other_metadata}
        db.metadata.insert_one(meta_data_dict)
